
--[[
--
-- If you wish to change which keystrokes the binder uses, modify the below
-- variables.  Remember that you will need to specify these keys inside WoW
-- as well.
--
--]]

local Binder_Key_1 = 'F6'
local Binder_Key_2 = 'F7'

--[[
--
-- Do not change anything below this line.
--
--]]

local CHORD_SEQ = {
    1 = {'ctrl',Binder_Key_1},
    2 = {'shift',Binder_Key_1},
    3 = {'alt',Binder_Key_1},
}
local CHORD_SPECIAL = {'ctrl','shift',Binder_Key_1}
local CHORD_FLOW = {Binder_Key_2}

local GKEY_MAP = {
     1 = {1,1,1},    2 = {1,1,2},    3 = {1,1,3},
     4 = {1,2,1},    5 = {1,2,2},    6 = {1,2,3},
     7 = {2,1,1},    8 = {2,1,2},    9 = {2,1,3},
    10 = {2,2,1},   11 = {2,2,2},   12 = {2,2,3},
    13 = {3,1,1},   14 = {3,1,2},   15 = {3,1,3},
    16 = {3,2,1},   17 = {3,2,2},   18 = {3,2,3},
}

function OnEvent(event, arg)
	OutputLogMessage("event = %s, arg = %s\n", event, arg);
    if event == "PROFILE_ACTIVATED" then
        OnProfileActivated()
    elseif event == "PROFILE_DEACTIVATED" then
        OnProfileDeactivated()
    elseif event == "G_PRESSED" then
        OnGPressed(arg)
    elseif event == "G_RELEASED" then
        OnGReleased(arg)
    elseif event == "M_PRESSED" then
        OnMPressed(arg)
    elseif event == "M_RELEASED" then
        OnMReleased(arg)
    else
        OutputLogMessage("Unknown event!")
    end
end

function OnProfileActivated()
    OutputLogMessage("Its' G15Binder: Driver started.")
end

function OnProfileDeactivated()
    OutputLogMessage("Its' G15Binder: Driver stopped.")
end

local key_down = nil
local chord_list = {}

function OnGPressed(n)
    -- If we are already pressing a key, block this one
    if key_down then return end
    key_down = true

    local m = GetMKeyState()
    local b,r,c = unpack(GKEY_MAP[n])
    
    -- Ok, clear any modifiers currently active
    ClearModifiers()

    -- Now we can transmit the sequence.
    table.clear(chord_list)
    chord_list[1] = CHORD_FLOW
    chord_list[2] = CHORD_SEQ[m]
    chord_list[3] = CHORD_SEQ[b]
    chord_list[4] = CHORD_SEQ[r]
    chord_list[5] = CHORD_SEQ[c]
    for _,chord in ipairs(chord_list) do
        PressAndReleaseChord(chord)
    end

    -- Restore modifiers for the final key
    RestoreModifiers()
    
    -- The final flow key is special: we press it, but do not release it
    PressChord(CHORD_FLOW)
end

function OnGReleased(n)
    -- No matter what sequence we are sending, we just need to release the
    -- flow control chord.
    ReleaseChord(CHORD_FLOW)
    key_down = false
end

function OnMPressed(n)
    -- If we are already pressing a key, block this one
    if key_down then return end
    key_down = true

    ClearModifiers()
    
    -- The M press sequence is very simple: FLOW, SPECIAL, (m), FLOW.
    table.clear(chord_list)
    chord_list[1] = CHORD_FLOW
    chord_list[2] = CHORD_SPECIAL
    chord_list[3] = CHORD_SEQ[n]
    for _,chord in ipairs(chord_list) do
        PressAndReleaseChord(chord)
    end

    RestoreModifiers()

    PressChord(CHORD_FLOW)
end

function OnMReleased(n)
    -- No matter what sequence we are sending, we just need to release the
    -- flow control chord.
    ReleaseChord(CHORD_FLOW)
    key_down = false
end

local MODIFIER_LIST = {'lalt','ralt','lshift','rshift','lctrl','rctrl'}
local stored_modifiers = {}

function ClearModifiers()
    if #stored_modifiers > 0 then
        OutputLogMessage("Error: "..(#stored_modifiers).." modifiers not restored!")
        OutputLCDMessage("Error: could not clear modifiers!")
    end
    
    for _,mod in ipairs(MODIFIER_LIST) do
        if IsModifierPressed(mod) then
            table.insert(stored_modifiers, mod)
            ReleaseKey(mod)
        end
    end
end

function RestoreModifiers()
    for _,mod in ipairs(stored_modifiers) do
        PressKey(mod)
    end
    
    table.clear(stored_modifiers)
end

function PressChord(chord)
    for _,key in ipairs(chord) do
        PressKey(key)
    end
end

local ReleaseChord_tt = {}
function ReleaseChord(chord)
    for _,key in ipairs(table.reverse(chord,ReleaseChord_tt)) do
        ReleaseKey(key)
    end
end

function PressAndReleaseChord(chord)
    PressChord(chord)
    ReleaseChord(chord)
end

function table.clear(t)
    while #t > 0 do table.remove(t) end
end

function table.reverse(t,dst)
    if dst == nil then
        dst = {}
    else
        table.clear(dst)
    end

    local lt = #t
    for i=lt,1,-1 do
        dst[lt-i+1] = t[i]
    end
    return dst
end
