
function ItsG15Binder_Nitpick(obj)

    --[[
    --
    -- Given two names, this function will attempt to locate the numerical
    -- portion of the names which are different, and will return a pattern,
    -- the first value, the last value, and the iteration direction (1 for
    -- forward, -1 for backwards.)
    --
    -- Formatting the pattern with the first value yields the first name, and
    -- formatting it with the last value yields the last name.
    --
    -- For instance:
    --
    -- Nitpick("Button3", "Button9") --> "Button%d", 3, 9, 1
    --
    -- Note: the total number of names in the sequence can be easily computed
    -- as: (last-first+1)*direction.
    --
    -- If the call fails for any reason, the first return value will be nil,
    -- with the second return value containing the reason for failure.
    --
    --]]
    function obj:Nitpick(firstName, lastName)
        -- The way we will do this is to simply look for *every* numerical
        -- component, and then compare them across the two names.  If we get
        -- an inconsistent number of numerical components, or if there is not
        -- exactly one numerical component that differs between the names,
        -- give up.
        
        if firstName == lastName then
            return nil, 'first and last names are the same'
        end

        -- First up, break the names into tables of alternating (name),
        -- (number) parts.
        local ft = {}
        local lt = {}
        for n,t in pairs{[firstName]=ft,[lastName]=lt} do
            for i,j in string.gmatch(n, "(%D*)(%d*)") do
                if i ~= '' then table.insert(t,i) end
                if j ~= '' then table.insert(t,j) end
            end
        end

        -- Sanity check: make sure that the tables are the same length, that
        -- all the name components are the same, and exactly one numerical
        -- component is different.
        if select('#',ft) ~= select('#',lt) then
            return nil, 'first and last names have different structures'
        end

        local ftl = #ft

        for i=1,ftl,2 do
            if ft[i] ~= lt[i] then
                return nil, 'first and last names have different bases'
            end
        end

        local nci -- numerical counter index

        do
           local c = 0 --difference count
           for i=2,ftl,2 do
               if ft[i] ~= lt[i] then
                   c = c + 1
                   nci = i
               end
           end
           if c ~= 1 then
               return nil, 'first and last names differ in more than one place'
           end
        end

        -- Ok, extract the first and last indices
        local fi = tonumber(ft[nci])
        local li = tonumber(lt[nci])

        -- Re-use the first name table to construct the pattern mask
        ft[nci] = '%d'
        local pat = table.concat(ft)

        -- Now work out the iteration direction
        local dir = nil
        if fi > li then dir = -1 else dir = 1 end

        -- All done!
        return pat, fi, li, dir
    end

end

