--[[

Its' G15 Binder - Main module
License: GPL v2 (no later version)

--]]

--[[

Todo:

* Finish documenting what the hell this does.
* Strip out legacy stuff that isn't used any more.

--]]

--[[

The problem with the Logitech G15 (and G11) keyboards are that the G keys are not real keys.  They can only ever act as stand-ins for other keys (or sequences of other keys.)  As a result, there's no way to directly bind G keys in-game, since WoW will only ever see them as normal keys.

To solve this, we exploit the keyboard profiler's macro capabilities so that each G key actually sends a sequence of key presses to WoW.  This sequence uniquely identifies the key being pressed.

Specifically, we want to be able to transmit two things from the keyboard to WoW:

1. G keys being pressed and
2. M keys being pressed.

Tackling the first, each G key can be uniquely represented by a set of four values in the form ``(m, b, r, c)``.  ``m`` is the mode (1-3,) ``b`` is the bank of six buttons (1-3 representing G1-G6, G7-G12 and G13-G18,) ``r`` is the row (1, 2) and ``c`` is the column (1-3.)

This means that we need at least three basic key combinations bound inside of WoW.  We will refer to these as sequence keystrokes 1, 2 and 3.

For the second, we'll introduce an out-of-band keystroke called 'special' or ``s``.  The mode can be communicated by prefixing either 1, 2 or 3 with ``s``.

Finally, we'll use a combined "execute and reset" keystroke called 'flow' or ``f``.  We want this for two reasons.  Firstly, by starting each sequence with it, we can reset the addon out of any erroneous state it might be in, as well as prevent the user from accidentally interfering.  Secondly, it allows us to more cleanly isolate the actual "dispatch" code into a single place.

The next issue is that we can't script any of the above due to the secure template system introduced in WoW 2.0.  Instead, we abuse the secure state driver to build a honking-great finite state machine.  It basically looks like this::

    (init)
        1, 2, 3 --> (m, ?, ?, ?)
            1, 2, 3 --> (m, b, ?, ?)
                1, 2, 3 --> (m, b, r, ?)
                    1, 2, 3 --> (m, b, r, c)
                        f --> (init), click bound button
                    f --> (init)
                f --> (init)
            f --> (init)
        s --> (mode ?)
            1, 2, 3 --> (mode m)
                f --> (init), process mode change
            f --> (init)
        f --> (init)
    
Implementing this in the secure templates system looks like this::

    SecureStateHeader   G15BinderHeader
        SecureButton    G15BinderSeq1
        SecureButton    G15BinderSeq2
        SecureButton    G15BinderSeq3
        SecureButton    G15BinderSpecial
        SecureButton    G15BinderFlow

The header keeps track of what state we're in, with the buttons describing the state transitions.  G15BinderFlow also serves to trigger the handling of G keys.

There also needs to be a layer of "filter" buttons between G15BinderFlow and the actual buttons to be "clicked."  Whilst you can code the action directly to the flow button, some action bars (such as Trinity) freak out when they get something other than the built-in buttons.  As such, this layer's only purpose is to change the button being clicked to the normal left mouse button, and then click the in-game action button.

Yes, it's confusing.  Just imagine having to work it all out from scratch... and then having to debug it!

--]]

--[[
--
-- Header stuff
--
--]]

local MAJOR_VERSION = "2.4"
local MINOR_VERSION = "1"
local REVISION = string.gsub("$Rev: 2$", "^.-(%d+).-$", "%1")

local L = AceLibrary"AceLocale-2.2":new"ItsG15Binder"

G15Binder = AceLibrary"AceAddon-2.0"
    :new("AceConsole-2.0","AceDB-2.0","AceEvent-2.0")
local G15Binder = G15Binder
--G15Binder.DEBUG = true

-- Load nitpick mixin
ItsG15Binder_Nitpick(G15Binder)

-- Addon information
G15Binder.title = "Its' G15 Binder"
G15Binder.version = string.format("%s.%s.%s",
    MAJOR_VERSION, MINOR_VERSION, REVISION)
G15Binder.date = string.gsub("$Date: 2008-07-26 01:15:03 -0500 (Sat, 26 Jul 2008) $",
    "^.-(%d%d%d%d-%d%d-%d%d).-$", "%1")

-- Load up libraries
local SM = AceLibrary"LibSharedMedia-2.0"
local Waterfall = AceLibrary"Waterfall-1.0"

-- Save global table
local _G = getfenv(0)

--[[
-- This is the inset between the Mode Border, um, border and the manually-created background
-- texture.
--]]
local INSET_SIZE = 4

--[[
--
-- Configuration
--
--]]
G15Binder:RegisterDB("ItsG15Binder_DB")
G15Binder:RegisterDefaults("profile", {
    keyBindings = {
        seq = 'F6',
        flow = 'F7',
    },
    clickBindings = {},
    borders = {
        enabled = false,
        padding = 5,
        color = {1,1,1,1},
        bgColor = {1,1,1,1},
        style = nil,
        bgStyle = nil,
        bgBlend = 'DISABLE',
    },
    paging = {
        enabled = false,
        pages = {1,2,3},
    },
    overlay = {
        enabled = false,
        format = 'g',
        anchor = 'TOPRIGHT',
        color = {0.3,0.3,1.0},
        scale = 1.0,
    },
})

local SEQ_PREFIX = {'CTRL-','SHIFT-','ALT-'}
local SPECIAL_PREFIX = 'CTRL-SHIFT-'

--[[
--
-- Stuff because I'm a lazy bastard
--
--]]

local function isnum(...)
    for i=1,select('#',...) do
        local x = select(i,...)
        if tonumber(x) == nil then return false end
    end
    return true
end

local function isnil(...)
    for i=1,select('#',...) do
        if select(i,...) ~= nil then return false end
    end
    return true
end

function G15Binder:OnInitialize()
    self:Debug("G15Binder:OnInitialize()")

    self.queuedBindings = {}
    self.attempts = 0

    self:CreateFrames()
    self:CreateBorderFrames()
    self:CreateOverlays()

    self:RegisterEvent("PLAYER_LOGIN")
end

function G15Binder:PLAYER_LOGIN()
    self:Debug("G15Binder:PLAYER_LOGIN()")

    self:BindKeys()
    self:BindButtons()

    local chatCmdTable = self:BuildCmdTable()

    self:RegisterChatCommand(L.ChatCommands, chatCmdTable)

    if Waterfall then
        Waterfall:Register(
            'G15BinderWaterfall',
            'title',L.WATERFALL_TITLE,
            'aceOptions',chatCmdTable,
            'hideTreeRoot',true,
            'colorR',0.3,
            'colorG',0.3,
            'colorB',1.0)
    end
end

function G15Binder:PostBindInit()
    self:SetBorderEnabled(self:GetBorderEnabled())
    self:SetPagingEnabled(self:GetPagingEnabled())
    self:SetKeyOverlayEnabled(self:GetKeyOverlayEnabled())
end

function G15Binder:BuildBorderList()
    return SM:List(SM.MediaType.BORDER)
end

function G15Binder:BuildBorderListUsage()
    return "{" .. table.concat(self:BuildBorderList(), " || ") .. "}"
end

function G15Binder:BuildBackgroundList()
    return SM:List(SM.MediaType.BACKGROUND)
end

function G15Binder:BuildBackgroundListUsage()
    return "{" .. table.concat(self:BuildBackgroundList(), " || ") .. "}"
end

local __BLEND_LIST = {'DISABLE', 'BLEND', 'ALPHAKEY', 'ADD', 'MOD'}

function G15Binder:BuildBlendList()
    return __BLEND_LIST
end

function G15Binder:BuildBlendListUsage()
    return "{" .. table.concat(self:BuildBlendList(), " || ") .. "}"
end

function G15Binder:BuildCmdTable()
    self:Debug("G15Binder:BuildCmdTable()")

    local function inject_config()
        self:Debug("G15Binder:BuildCmdTable.inject_config()")

        if not Waterfall then return nil end

        return
        {
            type = 'execute',
            name = L.CONFIG_NAME,
            desc = L.CONFIG_DESC,
            wfHidden = true,
            order = -1,
            func = function()
                Waterfall:Open('G15BinderWaterfall')
            end
        }
    end

    return
    {
        handler = G15Binder,
        type = "group",
        icon = [[Interface\AddOns\ItsG15Binder\icon]],
        args = {
            config = inject_config(),
            bind = {
                type = 'group',
                name = L.BIND_NAME,
                desc = L.BIND_DESC,
                args = {
                    mode = {
                        type = 'text',
                        name = L.BINDMODE_NAME,
                        desc = L.BINDMODE_DESC,
                        guiDesc = L.BINDMODE_DESC_GUI,
                        usage = L.BINDMODE_USAGE,
                        validate = L.BINDMODE_VALID,
                        get = 'GetBindingMode',
                        set = 'SetBindingMode',
                        order = 100,
                    },
                    _ = { type='header', order=109 },
                    clear = {
                        type = 'execute',
                        name = L.CLEAR_NAME,
                        desc = L.CLEAR_DESC,
                        order = 110,
                        confirm = L.CLEAR_CONFIRM,
                        confirmText = L.ARE_YOU_SURE,
                        func = 'ClearAllBindings',
                    },
                    keys = {
                        type = 'group',
                        name = L.KEYS_NAME,
                        desc = L.KEYS_DESC,
                        args = {
                            key1 = {
                                type = 'text',
                                name = L.KEY1_NAME,
                                desc = L.KEY1_DESC,
                                usage = L.KEY1_USAGE,
                                validate = 'ValidateKey',
                                get = 'GetKeyBinding',
                                set = 'SetKeyBinding',
                                passValue = 'seq',
                            },
                            key2 = {
                                type = 'text',
                                name = L.KEY2_NAME,
                                desc = L.KEY2_DESC,
                                usage = L.KEY2_USAGE,
                                validate = 'ValidateKey',
                                get = 'GetKeyBinding',
                                set = 'SetKeyBinding',
                                passValue = 'flow',
                            },
                        },
                    },
                },
            },
            display = {
                type = 'group',
                name = L.DISPLAY_NAME,
                desc = L.DISPLAY_DESC,
                args = {
                    overlay = {
                        type = 'group',
                        name = L.OVERLAY_NAME,
                        desc = L.OVERLAY_DESC,
                        groupType = 'inline',
                        args = {
                            enabled = {
                                type = 'toggle',
                                name = L.ENABLED_NAME,
                                desc = L.ENABLED_DESC,
                                order = 100,
                                get = 'GetKeyOverlayEnabled',
                                set = 'SetKeyOverlayEnabled',
                            },
                            format = {
                                type = 'text',
                                name = L.OVERLAY_FORMAT_NAME,
                                desc = L.OVERLAY_FORMAT_DESC,
                                usage = L.OVERLAY_FORMAT_USAGE,
                                order = 101,
                                validate = L.OVERLAY_FORMAT_VALID,
                                --validateDesc = L.OVERLAY_FORMAT_VALID_DESC,
                                get = 'GetKeyOverlayFormat',
                                set = 'SetKeyOverlayFormat',
                            },
                            anchor = {
                                type = 'text',
                                name = L.OVERLAY_ANCHOR_NAME,
                                desc = L.OVERLAY_ANCHOR_DESC,
                                order = 102,
                                validate = L.OVERLAY_ANCHOR_VALID,
                                --validateDesc = L.OVERLAY_ANCHOR_VALID_DESC,
                                get = 'GetKeyOverlayAnchor',
                                set = 'SetKeyOverlayAnchor',
                            },
                            color = {
                                type = 'color',
                                name = L.OVERLAY_COLOR_NAME,
                                desc = L.OVERLAY_COLOR_DESC,
                                order = 103,
                                get = 'GetKeyOverlayColor',
                                set = 'SetKeyOverlayColor',
                                hasAlpha = false,
                            },
                            scale = {
                                type = 'range',
                                name = L.OVERLAY_SCALE_NAME,
                                desc = L.OVERLAY_SCALE_DESC,
                                order = 104,
                                min = 0.10,
                                max = 1.50,
                                get = 'GetKeyOverlayScale',
                                set = 'SetKeyOverlayScale',
                            },
                        },
                    },
                },
            },
            mode = {
                type = 'group',
                name = L.MODE_NAME,
                desc = L.MODE_DESC,
                args = {
                    border = {
                        type = 'group',
                        name = L.BORDER_NAME,
                        desc = L.BORDER_DESC,
                        groupType = 'inline',
                        args = {
                            enabled = {
                                type = 'toggle',
                                name = L.ENABLED_NAME,
                                desc = L.ENABLED_DESC,
                                order = 1,
                                get = 'GetBorderEnabled',
                                set = 'SetBorderEnabled',
                            },
                            padding = {
                                type = 'range',
                                name = L.PADDING_NAME,
                                desc = L.PADDING_DESC,
                                order = 2,
                                min = 0,
                                max = 50,
                                get = 'GetBorderPadding',
                                set = 'SetBorderPadding',
                            },
                            style = {
                                type = 'text',
                                name = L.BORDER_STYLE_NAME,
                                desc = L.BORDER_STYLE_DESC,
                                order = 3,
                                get = 'GetBorderStyle',
                                set = 'SetBorderStyle',
                                validate = self:BuildBorderList(),
                                usage = self:BuildBorderListUsage(),
                            },
                            back = {
                                type = 'text',
                                name = L.BACK_STYLE_NAME,
                                desc = L.BACK_STYLE_DESC,
                                order = 4,
                                get = 'GetBackgroundStyle',
                                set = 'SetBackgroundStyle',
                                validate = self:BuildBackgroundList(),
                                usage = self:BuildBackgroundListUsage(),
                            },
                            color = {
                                type = 'color',
                                name = L.COLOR_NAME,
                                desc = L.COLOR_DESC,
                                order = 5,
                                get = 'GetBorderColor',
                                set = 'SetBorderColor',
                                hasAlpha = true,
                            },
                            bgcolor = {
                                type = 'color',
                                name = L.BG_COLOR_NAME,
                                desc = L.BG_COLOR_DESC,
                                order = 6,
                                get = 'GetBackgroundColor',
                                set = 'SetBackgroundColor',
                                hasAlpha = true,
                            },
                            bgblend = {
                                type = 'text',
                                name = L.BACK_BLEND_NAME,
                                desc = L.BACK_BLEND_DESC,
                                order = 7,
                                get = 'GetBackgroundBlend',
                                set = 'SetBackgroundBlend',
                                validate = self:BuildBlendList(),
                                usage = self:BuildBlendListUsage(),
                            },
                        },
                    },
                    page = {
                        type = 'group',
                        name = L.PAGE_NAME,
                        desc = L.PAGE_DESC,
                        groupType = 'inline',
                        args = {
                            enabled = {
                                type = 'toggle',
                                name = L.ENABLED_NAME,
                                desc = L.ENABLED_DESC,
                                get = 'GetPagingEnabled',
                                set = 'SetPagingEnabled',
                            },
                            mode1 = {
                                type = 'range',
                                name = L.MODE1_NAME,
                                desc = L.MODE1_DESC,
                                min = 1,
                                max = 6,
                                step = 1,
                                get = 'GetModePage',
                                set = 'SetModePage',
                                passValue = 1,
                            },
                            mode2 = {
                                type = 'range',
                                name = L.MODE2_NAME,
                                desc = L.MODE2_DESC,
                                min = 1,
                                max = 6,
                                step = 1,
                                get = 'GetModePage',
                                set = 'SetModePage',
                                passValue = 2,
                            },
                            mode3 = {
                                type = 'range',
                                name = L.MODE3_NAME,
                                desc = L.MODE3_DESC,
                                min = 1,
                                max = 6,
                                step = 1,
                                get = 'GetModePage',
                                set = 'SetModePage',
                                passValue = 3,
                            },
                        },
                    },
                },
            },
        },
    }
end

function G15Binder:MapState(m,b,r,c)
    if m == "init" and isnil(b,r,c) then
        return 0

    elseif m == 'error' and isnil(b,r,c) then
        return 1

    elseif m == 'special' and isnil(b,r,c) then
        return 'x'

    elseif m == 'mode' and isnum(b) and isnil(r,c) then
        return 'm'..b

    elseif isnum(m) and isnil(b,r,c) then
        return 1 + tonumber(m)

    elseif isnum(m,b) and isnil(r,c) then
        return 1 + 4 + 3*(tonumber(m)-1) + (tonumber(b)-1)

    elseif isnum(m,b,r) and isnil(c) then
        return 1 + 13 + 6*(tonumber(m)-1) + 2*(tonumber(b)-1)
            + (tonumber(r)-1)
    
    elseif isnum(m,b,r,c) then
        return 1 + 31 + 18*(tonumber(m)-1) + 6*(tonumber(b)-1)
            + 3*(tonumber(r)-1) + (tonumber(c)-1)

    else
        error(string.format(
            'G15Binder:MapState(%s,%s,%s,%s): bad arguments',
            tostring(m),tostring(b),tostring(r),tostring(c)))

    end
end

function G15Binder:MapName(m,b,r,c)
    if isnum(m,b) and isnil(r,c) then
        return 'M' .. m .. 'G' .. b

    elseif isnum(m,b,r,c) then
        return 'M' .. m
            .. 'G' .. self:MapNumber(b,r,c)

    elseif m == 'mode' and isnum(b) and isnil(r,c) then
        return 'M' .. b

    else
        error(string.format(
            'G15Binder:MapName(%s,%s,%s,%s): bad arguments',
            tostring(m),tostring(b),tostring(r),tostring(c)))

    end
end

function G15Binder:MapNumber(b,r,c)
    return (1 + 6*(tonumber(b)-1) + 3*(tonumber(r)-1)
                       + (tonumber(c)-1))
end

function G15Binder:DecodeButtonName(name)
    local m,g = string.match(name, 'M([123])G(1?[0-9])')

    if isnum(m,g) then
        return tonumber(m), tonumber(g)
    else
        error('Could not decode name "'..name'".')
    end
end

function G15Binder:GetHeader()
    return _G.G15BinderHeader
end

function G15Binder:GetSequenceButton(i)
    return _G['G15BinderSeq'..i]
end

function G15Binder:GetSpecialButton()
    return _G.G15BinderSpecial
end

function G15Binder:GetFlowButton()
    return _G.G15BinderFlow
end

function G15Binder:GetClickButton(m,b,r,c)
    if type(m) == 'string' and isnil(b,r,c) then
        return _G['G15BinderClick'..m]

    elseif isnum(m,b,r,c) then
        return _G['G15BinderClick'..self:MapName(m,b,r,c)]

    else
        error(string.format(
            'G15Binder:GetClickButton(%s,%s,%s,%s): bad arguments',
            tostring(m),tostring(b),tostring(r),tostring(c)))
    end
end

function G15Binder:GetModeButton(m)
    return _G['G15BinderMode'..m]
end

function G15Binder:GetBorderFrame(m)
    return _G['G15BinderBorder'..m]
end

function G15Binder:CreateBorderFrames()
    local bf = {}

    for i=1,3 do
        bf[i] = CreateFrame("Frame", "G15BinderBorder"..i, UIParent)
        
        local bft = bf[i]:CreateTexture("G15BinderBorderTexture"..i, "BACKGROUND")
        bf[i].texture = bft
        
        bft:ClearAllPoints()
        bft:SetPoint("TOPLEFT", bf[i], "TOPLEFT", INSET_SIZE, -INSET_SIZE)
        bft:SetPoint("BOTTOMRIGHT", bf[i], "BOTTOMRIGHT", -INSET_SIZE, INSET_SIZE)
        bft:Show()
        
        bf[i]:SetFrameStrata("BACKGROUND")
    end

    self:UpdateBorderStyle()
end

function G15Binder:UpdateBorderStyle()
    for i=1,3 do
        local bf = self:GetBorderFrame(i)
        bf:SetBackdrop(
        {
            --[[bgFile = SM:Fetch(SM.MediaType.BACKGROUND,
                self:GetBackgroundStyle()),]]
            bgFile = nil,
            --edgeFile = 'Interface\\Tooltips\\UI-Tooltip-Border',
            edgeFile = SM:Fetch(SM.MediaType.BORDER,
                self:GetBorderStyle()),
            tile = false,
            tileSize = 16,
            edgeSize = 16,
            insets =
            {
                top = INSET_SIZE,
                left = INSET_SIZE,
                right = INSET_SIZE,
                bottom = INSET_SIZE,
            },
        }
        )
        
        bf.texture:SetTexture(
            SM:Fetch(SM.MediaType.BACKGROUND, self:GetBackgroundStyle()))
        bf.texture:SetBlendMode(self:GetBackgroundBlend())
    end

    self:UpdateBorderColors()
end

function G15Binder:UpdateBorderColors()
    for i=1,3 do
        local bf = self:GetBorderFrame(i)
        --bf:SetBackdropColor(unpack(self.db.profile.borders.bgColor))
        bf.texture:SetVertexColor(unpack(self.db.profile.borders.bgColor))
        bf:SetBackdropBorderColor(unpack(self.db.profile.borders.color))
    end
end

function G15Binder:PositionBorderFrames()
    local cb = self.db.profile.clickBindings
    local b = self.db.profile.borders
    for m=1,3 do

        local bf = self:GetBorderFrame(m)
        local g1, g2

        for g=1,18 do
            if cb[self:MapName(m,g)] then
                g1 = g
                for h=18,g1,-1 do
                    if cb[self:MapName(m,h)] then
                        g2 = h
                        break
                    end
                end
                break
            end
        end

        if g1 and g2
                and _G[cb[self:MapName(m,g1)]]
                and _G[cb[self:MapName(m,g2)]] then
            self:Debug("Attaching border "..m.." to G"..g1.." and G"..g2)
            bf:ClearAllPoints()
            bf:SetPoint("TOPLEFT", _G[cb[self:MapName(m,g1)]], "TOPLEFT",
                -b.padding, b.padding)
            bf:SetPoint("BOTTOMRIGHT", _G[cb[self:MapName(m,g2)]], "BOTTOMRIGHT",
                b.padding, -b.padding)
        end

    end
end

function G15Binder:GetOverlayFrame(m,g)
    return _G['G15BinderOverlayFrameM'..m..'G'..g]
end

function G15Binder:GetOverlayText(m,g)
    return _G['G15BinderOverlayTextM'..m..'G'..g]
end

function G15Binder:CreateOverlays()
    for m=1,3 do
        for g=1,18 do
            local n = self:MapName(m,g)

            local f = CreateFrame('Frame',
                'G15BinderOverlayFrame'..n,
                UIParent)
            local t = f:CreateFontString(
                'G15BinderOverlayText'..n,
                'ARTWORK','NumberFontNormalSmallGray')
        end
    end
end

function G15Binder:UpdateOverlays()
    local cb = self.db.profile.clickBindings
    local o = self.db.profile.overlay

    local o_anchor = o.anchor
    local o_scale = o.scale
    local o_r,o_g,o_b = unpack(o.color)

    local o_format = L.OVERLAY_FORMAT[o.format]

    local overlayMap = {}

    for m=1,3 do
        for g=1,18 do

            local f = self:GetOverlayFrame(m,g)
            local t = self:GetOverlayText(m,g)

            if not self:GetKeyOverlayEnabled() then
                f:Hide()
            else
                local n = self:MapName(m,g)
                if cb[n] and not overlayMap[cb[n]] then
                    overlayMap[cb[n]] = n
                    local b = _G[cb[n]]
                    
                    if b then
                        f:SetParent(b)
                        f:ClearAllPoints()
                        f:SetPoint(o_anchor, b)
                        f:SetWidth(b:GetWidth())
                        f:SetHeight(b:GetHeight())
                        f:SetScale(o_scale)

                        t:ClearAllPoints()
                        t:SetPoint(o_anchor, f)
                        t:SetTextColor(o_r,o_g,o_b)
                        t:SetText(o_format(m,g))

                        f:Show()
                        t:Show()
                    end
                else
                    f:Hide()
                end
            end

        end
    end
end

function G15Binder:SetActiveMode(m)
   
    if not self.db.profile.borders.enabled then return end

    for i=1,3 do
        local bf = self:GetBorderFrame(i)

        if i == m then
            --self:GetBorderFrame(i):Show()
            UIFrameFadeIn(bf, 0.3*(1.0-bf:GetAlpha()), bf:GetAlpha(), 1.0)
        else
            --self:GetBorderFrame(i):Hide()
            UIFrameFadeOut(bf, 0.3*bf:GetAlpha(), bf:GetAlpha(), 0.0)
        end

    end

end

function G15Binder:CreateFrames()
    --[[
        The structure of the frames is like so:

        We have the top-level state driver, which keeps track of what state
        we're currently in.  This is called G15BinderHeader

        Then we have four child buttons: a, b, c & d.  A, b and c correspond to
        the components of each G key's four-dimensional "position."  The d button
        is used as the finaliser to terminate the sequence, and also detect
        possible problems.  These are called G15BinderSeqA through G15BinderSeqD.

        The states are named like so:

        init
            The initial state.  A,b,c map to (1,-,-,-), (2,-,-,-) and (3,-,-,-).
            D is an error.

        (m,-,-,-)
            This is the first part of the sequence, denoting the mode the key is
            in. A,b,c map to (m,1,-,-), (m,2,-,-), (m,3,-,-).  D is an error.

        (m,b,-,-)
            This indicates which bank of 6 the key is in.  A,b map to
            (m,b,1,-), (m,b,2,-).  C and d are errors.

        (m,b,r,-)
            This indicates which row of 3 the key is in.  A,b,c map to
            (m,b,r,1), (m,b,r,2), (m,b,r,3).  D is an error.

        (m,b,r,c)
            This specifies the exact G key being pressed by giving its mode, bank,
            row and column.  A,b,c are errors.  D causes the target button to be
            clicked, and then maps back to the init state.

        This means that there are a total of 1+3+3*3+3*3*2+3*3*2*3 = 85 states.
    ]]

    local sfh = CreateFrame("Frame", "G15BinderHeader", UIParent,
        "SecureStateHeaderTemplate")
    local sfs = {}
    for i=1,3 do
        sfs[i] = CreateFrame("Button", "G15BinderSeq"..i, UIParent,
            "SecureActionButtonTemplate")
        sfh:SetAttribute("addchild", sfs[i])
    end
    local sfx = CreateFrame("Button", "G15BinderSpecial", UIParent,
        "SecureActionButtonTemplate")
    sfh:SetAttribute('addchild', sfx)
    local sff = CreateFrame("Button", "G15BinderFlow", UIParent,
        "SecureActionButtonTemplate")
    sfh:SetAttribute('addchild', sff)

    local sfc = {}
    for m=1,3 do for b=1,3 do for r=1,2 do for c=1,3 do
        local n = self:MapName(m,b,r,c)
        sfc[n] = CreateFrame("Button", "G15BinderClick"..n, UIParent, 
            "SecureActionButtonTemplate")
        --sfh:SetAttribute('addchild', sfc[n])

        sfc[n].gName = n
        sfc[n]:SetAttribute('*statebutton*', '*:LeftButton')
        --sfc[n]:SetAttribute('*type*','click')
        sfc[n]:SetAttribute('type','macro')
        sfc[n]:SetAttribute('*unit*','none')
    end end end end

    local sfm = {}
    for m=1,3 do
        sfm[m] = CreateFrame("Button", "G15BinderMode"..m, UIParent,
            "SecureActionButtonTemplate")
        sfh:SetAttribute('addchild', sfm[m])

        sfm[m]:SetAttribute('*statebutton*', '*:LeftButton')
        sfm[m]:SetAttribute('*unit*','none')
    end

    -- Set the initial state, and other bits and pieces
    sfh:SetAttribute("state",self:MapState("init"))
    do
        local _self = self
        function sfh:StateChanged(ns)
            for i=1,3 do
                if ns == _self:MapState('mode',i) then
                    _self:SetActiveMode(i)
                end
            end
        end
    end

    for i=1,3 do sfs[i]:SetAttribute('unit','none') end

    sfx:SetAttribute('unit','none')

    sff:SetAttribute('type','click')
    sff:SetAttribute('unit','none')

    -- Create the state transitions
    do
        local function ms(...) return self:MapState(...) end
        local function mn(...) return self:MapName(...) end

        -- Sequence state changes
        for s=1,3 do -- seq. button
            local trspec = {}
            local function tsa(s,d) table.insert(trspec, s..':'..d) end

            tsa(ms('init'), ms(s))

            for m=1,3 do -- mode
                tsa(ms(m), ms(m,s))

                for b=1,3 do -- bank
                    if s < 3 then
                        tsa(ms(m,b), ms(m,b,s))
                    end

                    for r=1,2 do -- row
                        tsa(ms(m,b,r), ms(m,b,r,s))
                    end
                end
            end

            -- Do special transitions
            tsa(ms'special',ms('mode',s))

            -- All other transitions are illegal
            tsa('*', ms'error')

            sfs[s]:SetAttribute("newstate", table.concat(trspec,';'))
        end

        -- Special state changes
        do
            local trspec = {}
            local function tsa(s,d) table.insert(trspec, s..':'..d) end

            -- From the init state, we go into the special state; that's it
            tsa(ms'init', ms'special')

            -- All other transitions are illegal
            tsa('*', ms('error'))

            sfx:SetAttribute("newstate", table.concat(trspec,';'))
        end

        -- Flow state changes
        do
            local trspec = {}
            local function tsa(s,d) table.insert(trspec, s..':'..d) end

            local remap = {}
            local function rem(s,b) table.insert(remap, s..':'..b) end

            -- While we're here, we're going to do the button remapping
            for m=1,3 do
                for b=1,3 do
                    for r=1,2 do
                        for c=1,3 do
                            local n = mn(m,b,r,c)
                            rem(ms(m,b,r,c), n)
                            sff:SetAttribute('*clickbutton-'..n, sfc[n])
                        end
                    end
                end
            end

            for m=1,3 do
                local n = mn('mode',m)
                rem(ms('mode',m), n)
                sff:SetAttribute('*clickbutton-'..n, sfm[m])
            end

            -- All transitions just go back to the init state
            tsa('*', ms('init'))
            rem('*', 'reset')

            sff:SetAttribute('newstate', table.concat(trspec,';'))
            sff:SetAttribute('statebutton', table.concat(remap,';'))
        end
    end
end

function G15Binder:BindKeys()
    self:Debug('G15Binder:BindKeys()')

    local kb = self.db.profile.keyBindings
    
    for i=1,3 do
        local k = SEQ_PREFIX[i]..kb.seq
        local n = self:GetSequenceButton(i):GetName()
        self:Debug(' ['..k..'] --> '..n)
        SetBindingClick(k, n)
    end
    do
        local k = SPECIAL_PREFIX..kb.seq
        local n = self:GetSpecialButton():GetName()
        self:Debug(' ['..k..'] --> '..n)
        SetBindingClick(k, n)
    end
    do
        local k = kb.flow
        local n = self:GetFlowButton():GetName()
        self:Debug(' ['..k..'] --> '..n)
        SetBindingClick(k, n)
    end
end

function G15Binder:BindButtons()
    self:Debug('G15Binder:BindButtons()')
    local fb = self:GetFlowButton()
    local queued = false
    for m=1,3 do
    for b=1,3 do
    for r=1,2 do
    for c=1,3 do
        local n = self:MapName(m,b,r,c)
        local tn = self.db.profile.clickBindings[n]
        local cb = self:GetClickButton(m,b,r,c)
        if tn then
            local target = _G[tn]
            if target then
                self:Debug('  |cff8888ff'..n..'|r -> |cff88ff88'..tn..'|r')
                --cb:SetAttribute('*clickbutton-'..n, target)
                cb:SetAttribute('*macrotext*', '/click '..tn)
            else
                self:Debug('  |cff8888ff'..n..'|r -> |cffff8888'..tn..'|r')
                self:QueueBind(cb, tn)
                queued = true
            end
        else
            cb:SetAttribute('*macrotext*', nil)
        end

    end
    end
    end
    end

    fb.callback = function(self, unit, button)
        G15Binder:FlowCallback(self, unit, button)
    end

    if not queued then
        self:PostBindInit()
    end
end

function G15Binder:SetCallbackMode(enable)
    if enable then
        self:GetFlowButton():SetAttribute('type','callback')
    else
        self:GetFlowButton():SetAttribute('type','click')
    end
end

function G15Binder:GetCallbackMode()
    return (self:GetFlowButton():GetAttribute'type' == 'callback')
end

--[[
function G15Binder:SetTrace()
    local header = self:GetHeader()
    local g15binder = self
    function header:StateChanged(newstate)
        g15binder:Print('--> |cffff88ff'..newstate..'|r')
    end
end

function G15Binder:ClearTrace()
    self:GetHeader().StateChanged = nil
end
--]]

function G15Binder:IsGButton(button)
    return string.match(button, 'M[123]G1?[0-9]')
end

function G15Binder:FlowCallback(s, unit, button)
    self:Debug('[|cff8888ff'..button..'|r]')
    
    if self.callbackAction == 'bind-single' then

        if self:IsGButton(button) then
            -- Get the button under the mouse, and bind that G key to it
            local b = GetMouseFocus()
            if not b then return end

            self:SetBinding(button, b:GetName())
        end

    elseif self.callbackAction == 'bind-range-first' then

        if self:IsGButton(button) then
            local b = GetMouseFocus()
            if not b then return end

            if not b.Click then
                self:Print(L.ERR_NOT_BUTTON, b:Name())
                return
            end

            self:Print(L.BINDING_RANGE_START, button, b:GetName())

            self.callbackAction = 'bind-range-second'
            self.bindRangeFirstKey = button
            self.bindRangeFirstButton = b:GetName()
        end

    elseif self.callbackAction == 'bind-range-second' then

        if self:IsGButton(button) then
            local b = GetMouseFocus()
            if not b then return end

            local m,g1 = self:DecodeButtonName(self.bindRangeFirstKey)
            local _,g2 = self:DecodeButtonName(button)
            local n1 = self.bindRangeFirstButton
            local n2 = b:GetName()

            self:BindRange(m,g1,g2,n1,n2)

            self.callbackAction = 'bind-range-first'
        end

    end
end

function G15Binder:BindRange(m,g1,g2,firstName,lastName)
    if g2 < g1 then
        local t = g1
        g1 = g2
        g2 = t
    end

    self:Print(L.BINDING_RANGE, m, g1, m, g2, firstName, lastName)

    local nm,n1,n2,dir = self:Nitpick(firstName, lastName)
    if nm == nil then
        self:Print(L.ERR_GENERAL, n1)
        return
    end

    -- Check lengths are consistent
    if (g2-g1+1) ~= (n2-n1+1)*dir then
        self:Print(L.ERR_DIFF_NUM, (g2-g1+1), (n2-n1+1)*dir)
        return
    end

    -- Ok; do the bind
    for i=0,g2-g1 do
        local g = self:MapName(m,g1+i)
        local n = string.format(nm,n1+i*dir)
        self:SetBinding(g,n)
    end

    self:Print(L.DONE)
end

function G15Binder:SetBinding(g, name)
    local cb = self.db.profile.clickBindings

    if cb[g] then
        self:Print(L.BINDING_KEY_REPL, g, name, cb[g])
    
    else
        self:Print(L.BINDING_KEY, g, name)

    end

    if _G[name].Click == nil then
        self:Print(L.ERR_NOT_BUTTON, name)
    else
        cb[g] = name
        --self:GetClickButton(g):SetAttribute("*clickbutton*", _G[name])
        self:GetClickButton(g):SetAttribute('*macrotext*',
            '/click '..name)

        self:PositionBorderFrames()
        self:UpdateOverlays()
    end
end

function G15Binder:QueueBind(button, targetName)
    self:Debug("Queued "..button:GetName().." -> "..targetName)
    self.queuedBindings[button] = targetName

    if self.queueFrame == nil then
        self.queueFrame = CreateFrame('Frame','G15BinderQueueFrame',UIParent)
    end
    self.queueFrame:SetScript('OnUpdate', function() self:ProcessQueue() end)
end

function G15Binder:ProcessQueue()
    self:Debug("G15Binder:ProcessQueue()")

    self.queueFrame:SetScript('OnUpdate',nil)

    local queue = self.queuedBindings
    self.queuedBindings = {}

    for cb,tn in pairs(queue) do
        local t = _G[tn]
        if t then
            self:Debug('  |cff0000ff'..cb:GetName()..'|r')
            self:Debug('  |cff8888ff'..cb.gName..'|r -> |cff88ff88'..tn..'|r')
            --cb:SetAttribute('*clickbutton*', t)
            cb:SetAttribute('*macrotext*', '/click '..tn)
        else
            self:Debug('|cffff0000Error|r: could not bind '
                ..cb.gName..' to '..tn..'.')
        end
    end

    self:PostBindInit()
end

function G15Binder:EnableBorders()
    self:PositionBorderFrames()
    for i=1,3 do
        self:GetBorderFrame(i):Show()
        self:GetBorderFrame(i):SetAlpha(1)
    end
end

function G15Binder:DisableBorders()
    for i=1,3 do self:GetBorderFrame(i):Hide() end
end

function G15Binder:EnablePaging()
    for i=1,3 do
        self:GetModeButton(i):SetAttribute('*type*', 'actionbar')
    end
    self:UpdateModePages()
end

function G15Binder:DisablePaging()
    for i=1,3 do
        self:GetModeButton(i):SetAttribute('*type*', nil)
    end
end

function G15Binder:UpdateModePages()
    for i=1,3 do
        self:GetModeButton(i):SetAttribute('*action*',
            self.db.profile.paging.pages[i])
    end
end

function G15Binder:Debug(...)
    if self.DEBUG then self:Print(...) end
end

function G15Binder:EnableSingleBinding()
    self:SetCallbackMode(true)
    self.callbackAction = 'bind-single'

    self:Print(L.SINGLE_BINDING_ENABLED)
end

function G15Binder:EnableRangeBinding()
    self:SetCallbackMode(true)
    self.callbackAction = 'bind-range-first'

    self:Print(L.RANGE_BINDING_ENABLED)
end

function G15Binder:GetBindingMode()
    return self.bindingMode or 'off'
end

function G15Binder:SetBindingMode(m)
    self.bindingMode = m

    if m == 'off' then
        self:DisableBinding()
    elseif m == 'single' then
        self:EnableSingleBinding()
    elseif m == 'range' then
        self:EnableRangeBinding()
    end
end

function G15Binder:ClearAllBindings()
    self.db.profile.clickBindings = {}
    self:BindButtons()

    self:Print(L.BINDINGS_CLEARED)
end

function G15Binder:DisableBinding()
    self:SetCallbackMode(false)

    self:Print(L.BINDING_DISABLED)
end

function G15Binder:GetBorderEnabled()
    return self.db.profile.borders.enabled
end

function G15Binder:SetBorderEnabled(v)
    self.db.profile.borders.enabled = v
    if v then
        self:EnableBorders()
    else
        self:DisableBorders()
    end
end

function G15Binder:GetBorderPadding()
    return self.db.profile.borders.padding
end

function G15Binder:SetBorderPadding(v)
    self.db.profile.borders.padding = v
    self:PositionBorderFrames()
end

function G15Binder:GetBorderColor()
    return unpack(self.db.profile.borders.color)
end

function G15Binder:SetBorderColor(r,g,b,a)
    self.db.profile.borders.color = {r,g,b,a}
    self:UpdateBorderColors()
end

function G15Binder:GetBackgroundColor()
    return unpack(self.db.profile.borders.bgColor)
end

function G15Binder:SetBackgroundColor(r,g,b,a)
    self.db.profile.borders.bgColor = {r,g,b,a}
    self:UpdateBorderColors()
end

function G15Binder:GetPagingEnabled()
    return self.db.profile.paging.enabled
end

function G15Binder:SetPagingEnabled(v)
    self.db.profile.paging.enabled = v
    if v then
        self:EnablePaging()
    else
        self:DisablePaging()
    end
end

function G15Binder:GetModePage(mode)
    return self.db.profile.paging.pages[mode]
end

function G15Binder:SetModePage(mode, v)
    self.db.profile.paging.pages[mode] = v
    self:UpdateModePages()
end

function G15Binder:ValidateKey()
    return true
end

function G15Binder:GetKeyBinding(binding)
    return self.db.profile.keyBindings[binding]
end

function G15Binder:SetKeyBinding(binding, key)
    self.db.profile.keyBindings[binding] = key
    self:BindKeys()
end

function G15Binder:GetBorderStyle()
    local b = self.db.profile.borders
    if b.style then
        return b.style
    else
        return self:BuildBorderList()[1]
    end
end

function G15Binder:SetBorderStyle(s)
    self.db.profile.borders.style = s
    self:UpdateBorderStyle()
end

function G15Binder:GetBackgroundStyle()
    local b = self.db.profile.borders
    if b.bgStyle then
        return b.bgStyle
    else
        --return self:BuildBackgroundList()[1]
        return 'None'
    end
end

function G15Binder:SetBackgroundStyle(s)
    self.db.profile.borders.bgStyle = s
    self:UpdateBorderStyle()
end

function G15Binder:GetBackgroundBlend()
    local b = self.db.profile.borders
    if b.bgBlend then
        return b.bgBlend
    else
        return 'DISABLED'
    end
end

function G15Binder:SetBackgroundBlend(v)
    self.db.profile.borders.bgBlend = v
    self:UpdateBorderStyle()
end

function G15Binder:GetKeyOverlayEnabled()
    return self.db.profile.overlay.enabled
end

function G15Binder:SetKeyOverlayEnabled(v)
    self.db.profile.overlay.enabled = v
    self:UpdateOverlays()
end

function G15Binder:GetKeyOverlayFormat()
    return self.db.profile.overlay.format
end

function G15Binder:SetKeyOverlayFormat(v)
    self.db.profile.overlay.format = v
    self:UpdateOverlays()
end

function G15Binder:GetKeyOverlayAnchor()
    return self.db.profile.overlay.anchor
end

function G15Binder:SetKeyOverlayAnchor(v)
    self.db.profile.overlay.anchor = v
    self:UpdateOverlays()
end

function G15Binder:GetKeyOverlayColor()
    return unpack(self.db.profile.overlay.color)
end

function G15Binder:SetKeyOverlayColor(r,g,b)
    self.db.profile.overlay.color = {r,g,b}
    self:UpdateOverlays()
end

function G15Binder:GetKeyOverlayScale()
    return self.db.profile.overlay.scale
end

function G15Binder:SetKeyOverlayScale(v)
    self.db.profile.overlay.scale = v
    self:UpdateOverlays()
end